<?php

namespace HCM\AVL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;

class AvaliacaoController extends Crud {

    protected $tabelaFolhaService;
    protected $dataSourceName;
    protected $dataSourceAvaliacao = 'avl_avaliacaogeral';
    protected $dataSourceAvaliacaoGet = 'getAvaliacaoGeral';
    protected $dataSourceQuestAvaliacao = 'avl_questavaliacao';
    protected $dataSourceQuestAvaliacaoGet = 'getQuestAvaliacao';
    protected $dataSourceParticipante = 'avl_participaval';
    protected $dataSourceParticipanteGet = 'getParticipante';
    protected $dataSourceCombinacao = 'avl_combquestaval';
    protected $dataSourceCombinacaoGet = 'getCombAval';
    protected $dataSourceAgrupamento = 'avl_agrupquestaval';
    protected $dataSourceAgrupamentoGet = 'getAgrupAval';
    protected $dataSourceTipoRelac = 'avl_tiporelacaval';
    protected $dataSourceTipoRelacGet = 'getTipoRelacAval';
    protected $dataSourceParcNegAval  = 'avl_parcnegocio_avaliacao';
    protected $dataSourceParcNegAvalGet  = 'getParcNegocioAval';
    protected $dataSourceCargoAvaliacaoGet = 'getCargoAvaliacao';
    protected $dataSourceVinculoAvaliacao_get = 'getVinculoAvaliacao';

    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRAVALIACAO', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\AVL\Services\AvaliacaoService $AvaliacaoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->AvaliacaoService = $AvaliacaoService;
        parent::__construct($dataSourceManager);
    }

    public function getAvaliacaoGeral(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceAvaliacaoGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setAvaliacaoGeral(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRAVALIACAO', $row)) {
                    if ($row['NRAVALIACAO'] === '') {
                        $row['NRAVALIACAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'AVALIACAOGERAL', 1, 22, true);
                    }
                } else {
                    $row['NRAVALIACAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'AVALIACAOGERAL', 1, 22, true);
                }
                $row['NRCRITERIORESUL'] = isset($row['NRCRITERIORESUL']) ? $row['NRCRITERIORESUL'] : NULL;
                $row['NRNIVELAVALIA'] = isset($row['NRNIVELAVALIA']) ? $row['NRNIVELAVALIA'] : NULL;
            }
            
             $this->dataSourceName = $this->dataSourceAvaliacao;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteAvaliacaoGeral(DTO\Request\DataSet $request, DTO\Response $response){
        try{
        foreach ($request->getDataSet()->getRows() as &$row) {

            $this->dataSourceName = $this->dataSourceAvaliacao;   
            $questExists = $this->AvaliacaoService->questaoExists($row['NRORG'],$row['NRAVALIACAO']);
            $agrupExists = $this->AvaliacaoService->agrupExists($row['NRORG'],$row['NRAVALIACAO']);
            $combExists = $this->AvaliacaoService->combExists($row['NRORG'],$row['NRAVALIACAO']);
            $relacExists = $this->AvaliacaoService->relacExists($row['NRORG'],$row['NRAVALIACAO']);
            $participaExists = $this->AvaliacaoService->participanteExists($row['NRORG'],$row['NRAVALIACAO']);
        
            if($questExists || $combExists || $agrupExists || $relacExists || $participaExists){
                $Message = "";
                $exist_array = array($questExists,$combExists,$agrupExists,$relacExists,$participaExists);
                $atr_array = array("Questoes","Combinacoes","Agrupamentos","Relacionamentos","Participantes");
                $index = 0;
                foreach($exist_array as $exist){
                    if($exist){
                        $Message .= " ".$atr_array[$index].",";
                    }
                    $index++;
                }
                $Message = rtrim($Message,",");
                $Message = "Existem".$Message." afiliadas a esta avaliação,<BR> favor deleta-las antes de exclui-la";
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($Message,'E'));
            }else{
                parent::delete($request, $response);
            }
        }    
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
        

    }
    
    public function importarPadraAval(DTO\Request\DataSet $request, DTO\Response $response){
        try{
        foreach ($request->getDataSet()->getRows() as &$row) {

            $this->dataSourceName = $this->dataSourceAvaliacao;   
            $agrupExists = $this->AvaliacaoService->agrupExists($row['NRORG'],$row['NRAVALIACAO']);
            $combExists = $this->AvaliacaoService->combExists($row['NRORG'],$row['NRAVALIACAO']);
            $questExists = $this->AvaliacaoService->questaoExists($row['NRORG'],$row['NRAVALIACAO']);
            $relacExists = $this->AvaliacaoService->relacExists($row['NRORG'],$row['NRAVALIACAO']);
            
            if($questExists || $combExists || $agrupExists || $relacExists ){
                $Message = "";
                $exist_array = array($questExists,$combExists,$agrupExists,$relacExists);
                $atr_array = array("Questoes","Combinacoes","Agrupamentos","Relacionamentos");
                $index = 0;
                foreach($exist_array as $exist){
                    if($exist){
                        $Message .= " ".$atr_array[$index].",";
                    }
                    $index++;
                }
                $Message = rtrim($Message,",");
                $Message = "Existem".$Message." afiliadas a esta avaliação,<BR> favor deleta-las antes de importar";
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($Message,'E'));
            }else{
                $this->AvaliacaoService->importarPadraAval($response, $row['NRORG'],$row['NRAVALIACAO'],$row['NRPADRAVALIACAO']);
            }
        }    
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
        

    }
    
    public function liberarAvaliacao(DTO\Request\DataSet $request, DTO\Response $response){
        
        try{
            foreach ($request->getDataSet()->getRows() as &$row) {
            $nravaliacao = $row['NRAVALIACAO'];
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $combNaoLiberada = array();
            $params = (array(
            "P_NRAVALIACAO" => $nravaliacao,
            "P_NRORG" => $nrorg,
            "P_NRCOMBQUESTAVAL" => 1
            ) );
            //Filtra os participantes(relacionamentos) que nao foram liberados. Consulta base para gravar a liberacao na execucao
            $combQuestAval = $this->AvaliacaoService->quantidadeCombAval($nravaliacao, $nrorg);
            $combinacao = array();
            foreach($combQuestAval as &$comb){
                $combSemQuest = $this->AvaliacaoService->questCombinacao($nravaliacao, $comb['NRCOMBQUESTAVAL']);
                if(empty($combSemQuest)){
                    $combNaoLiberada = $comb;
                    $params['P_NRCOMBQUESTAVAL'] = $combNaoLiberada['NRCOMBQUESTAVAL'];
                }
            }
            $consultaParticipantesLibera = $this->AvaliacaoService->consultaParticipantesLibera($nravaliacao, $nrorg);
            //Seleciona participantes novos a serem inseridos na execucao da avaliacao
            //$contaParticipantes = $this->AvaliacaoService->contaParticipantes($params);
    
            //Retorna a quantidade de execucoes a avaliacao setada possui
            
            if($consultaParticipantesLibera != null){
                foreach($consultaParticipantesLibera as $value){
        	    //Insere execucao da avaliacao vazia
                    if(empty($value['NRCOMBQUESTAVAL'])){
                            $nrcombquestaval = 1;
                    }else{
                            $nrcombquestaval = $value['NRCOMBQUESTAVAL'];
                    }
                    
                    $nrexecavaliacao = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'EXECAVALIACAO', 1, 22, true);
                    $this->AvaliacaoService->InsereExecavaliacao($nrexecavaliacao, $nrorg, $value['NRAVALIACAO'], $value['NRPARCAVALIADO'], $value['NRPARCEXECAVAL'],
                                                                    $value['NRQUESTAO'], $nrcombquestaval, null, null, null, null, null, null, 'SISTEMA');
        	
                    
                }//Fim ForEach
                $Message = "Avaliação liberada para participante(s)!";
                if($combNaoLiberada){
                    $Message = "Combinação '".$combNaoLiberada['NMCOMBQUESTAVAL']."' sem questões! Não foi possível liberar avaliação para seus participantes. <br>".$Message;
                }
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($Message,'S'));
            }else{
                $Message = "Não há participante(s) para liberar avaliação";
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($Message,'W'));
            }
            }    
        }catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function importarParticipantes(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                switch($row['IDORDEM']){
                    case 'ALFABETICA':
                        $dsinicio = $row['NMVINCULOM_INI'];
                        $dsfim = $row['NMVINCULOM_FIN'];
                        break;
                    case 'MATRICULA':
                        $dsinicio = $row['VRMATRICULA_INI'];
                        $dsfim = $row['VRMATRICULA_FIN'];
                        break;
                    case 'DATA_ADMISSAO':
                        $dsinicio = $row['DTADMISS_INI'];
                        $dsfim = $row['DTADMISS_FIN'];
                        break;
                    case 'ESTRUTURA':
                        $dsinicio = $row['NR_ESTRUTURA_LEG'];
                        $dsfim = $row['NR_ESTRUTURA_GER'];
                        break;
                }
                $nrparceiro = isset($row['NRPARCNEGOCIO']) ? $row['NRPARCNEGOCIO'] : NULL;
                $row['NROCUPACAOM'] = isset($row['NROCUPACAOM']) ? $row['NROCUPACAOM'] : NULL;
                $resp = $this->AvaliacaoService->importarParticipante($row['NRAVALIACAO'],$row['NRTIPOHIERARQORG'],$row['NRTIPORELACIONA'], $row['NROCUPACAOM'], $row['IDORDEM'],$dsinicio, $dsfim, $this->hcmParameters->getNrorgTrabalho(), $nrparceiro,$this->hcmParameters->getDtCompetencia(),0);
                foreach($resp as $r){
                        $response->addNotification(new \Zeedhi\Framework\DTO\Response\Notification($r[1],$r[0]));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getCargoAvaliacao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceCargoAvaliacaoGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getVinculoAvaliacao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceVinculoAvaliacao_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getQuestAvaliacao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $this->AvaliacaoService->InsereCombPadrao();
        $filterData->setDataSourceName($this->dataSourceQuestAvaliacaoGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getParcNegocioAval(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceParcNegAvalGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setQuestAvaliacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['NRCOMBQUESTAVAL'] = isset($row['NRCOMBQUESTAVAL']) ? $row['NRCOMBQUESTAVAL'] : NULL;
                $row['NRAGRUPQUESTAVA'] = isset($row['NRAGRUPQUESTAVA']) ? $row['NRAGRUPQUESTAVA'] : NULL;
                $row['NRNIVELQUESTAO'] = isset($row['NRNIVELQUESTAO']) ? $row['NRNIVELQUESTAO'] : NULL;
                $row['NRQUESTAO'] = isset($row['NRQUESTAO']) ? $row['NRQUESTAO'] : NULL;
                $row['NRTEMAANALISE'] = isset($row['NRTEMAANALISE']) ? $row['NRTEMAANALISE'] : NULL;
                
                if (array_key_exists('NRQUESTAVALIA', $row)) {
                    if ($row['NRQUESTAVALIA'] === '') {
                        $row['NRQUESTAVALIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTAVALIA', 1, 22, true);
                    }}
                else {
                    $row['NRQUESTAVALIA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTAVALIA', 1, 22, true);
                }
                if($row['IDSORTQUESTAVAL'] != 'SEM_SORTEIO'){
                    $row['NRQUESTAO'] = $this->AvaliacaoService->getQuestaoSorteio($row['NRORG'],$row['NRNIVELQUESTAO'],$row['IDSORTQUESTAVAL']);
                    if($row['NRQUESTAO'] == NULL){
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Nenhuma questão com estes parametros encontrada","E"));
                    }else{
                        $this->dataSourceName = $this->dataSourceQuestAvaliacao;
                        parent::save($request, $response);
                    }
                }else{
                    $this->dataSourceName = $this->dataSourceQuestAvaliacao;
                    parent::save($request, $response);
                }
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteQuestAvaliacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceQuestAvaliacao;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    
    public function getParticipante(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceParticipanteGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setParticipante(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row){
                if (array_key_exists('NRPARTICIPAVAL', $row)) {
                    if ($row['NRPARTICIPAVAL'] === '') {
                        $row['NRPARTICIPAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PARTICIPAVAL', 1, 22, true);
                    }
                } else {
                    $row['NRPARTICIPAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PARTICIPAVAL', 1, 22, true);
                }
                $row['CDTIPORESU'] = isset($row['CDTIPORESU']) ? $row['CDTIPORESU'] : NULL;
                $combquestaval = $this->AvaliacaoService->getCombinacao($row['NRORG'], $row['NRAVALIACAO']);
                $row['NRCOMBQUESTAVAL'] = $combquestaval[0]['NRCOMBQUESTAVAL'];
            }   
            $this->dataSourceName = $this->dataSourceParticipante;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteParticipante(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceParticipante;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    
    public function getCombAval(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceCombinacaoGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setCombAval(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRCOMBQUESTAVAL', $row)) {
                    if ($row['NRCOMBQUESTAVAL'] === '') {
                        $row['NRCOMBQUESTAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'COMBQUESTAVAL', 1, 22, true);
                    }
                } else {
                    $row['NRCOMBQUESTAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'COMBQUESTAVAL', 1, 22, true);
                }
            }
            $this->dataSourceName = $this->dataSourceCombinacao;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteCombAval(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceCombinacao;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    
    public function getAgrupAval(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceAgrupamentoGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setAgrupAval(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRAGRUPQUESTAVA', $row)) {
                    if ($row['NRAGRUPQUESTAVA'] === '') {
                        $row['NRAGRUPQUESTAVA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'AGRUPQUESTAVAL', 1, 22, true);
                    }
                } else {
                    $row['NRAGRUPQUESTAVA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'AGRUPQUESTAVAL', 1, 22, true);
                }
            }
            $this->dataSourceName = $this->dataSourceAgrupamento;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteAgrupAval(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceAgrupamento;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    
    public function getTipoRelacAval(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceTipoRelacGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setTipoRelacAval(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRTIPORELACAVAL', $row)) {
                    if ($row['NRTIPORELACAVAL'] === '') {
                        $row['NRTIPORELACAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'TIPORELACAVAL', 1, 22, true);
                    }
                } else {
                    $row['NRTIPORELACAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'TIPORELACAVAL', 1, 22, true);
                }
            if($this->AvaliacaoService->validaTipoRelac($row['NRORG'],$row['NRAVALIACAO'],$row['NRTIPORELACAVAL'],$row['NRTIPORELACIONA'])){
                $this->dataSourceName = $this->dataSourceTipoRelac;
                parent::save($request, $response);
            }else{
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Este tipo de relacionamento já existe neste questionario",'E'));
            }}
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteTipoRelacAval(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceTipoRelac;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    
    public function getQuestAval(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceQuestaval);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    
    public function cadastroAutomaticoParticipantes(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $avaliacao = $row['NRAVALIACAO'];
                $avaliador = $row['NRPARCEXECAVAL'];
                $list_avaliado = $row['NRAVALIADOS_LIST'];
                $relacionamento = $row['NRTIPORELACIONA'];
                $idSituacao = isset($row['IDSITUACAOAVAL']) ? $row['IDSITUACAOAVAL'] : "PENDENTE";
                $resulgeral = isset($row['VRRESULTAVALIA']) ? $row['VRRESULTAVALIA'] : NULL;
                $resulnom = isset($row['CDTIPORESU']) ? $row['CDTIPORESU'] : NULL;
                $dtconcl = isset($row['DTCONCLUSAOAVAL']) ? $row['DTCONCLUSAOAVAL'] : NULL;
                $idAtivo = $row['IDATIVO'];
                $this->AvaliacaoService->cadastroAutomatico($avaliacao,$avaliador,$list_avaliado,$relacionamento,$idSituacao,$resulgeral,$resulnom,$dtconcl,$idAtivo);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function novaQuestao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $nrnivelpadquest = $row['NRNIVELPADQUEST'];
                $txquestao       = $row['TXQUESTAO'];
                $idtipoquestao   = $row['IDTIPOQUESTAO'];
                $idquestaoobriga = $row['IDQUESTAOOBRIGA'];
                $idopcaoresposta = $row['IDOPCAORESPOSTA'];
                $dtrevisaquestao = isset($row['DTREVISAQUESTAO']) ? $row['DTREVISAQUESTAO'] : NULL;
                $idativo = $row['IDATIVO'];
                $return = array();
                $return = $this->AvaliacaoService->novaQuestao($nrnivelpadquest,$txquestao,$idtipoquestao,$dtrevisaquestao,$idquestaoobriga,$idopcaoresposta,$idativo);
                $response->addDataSet(new DataSet("cod",$return));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    
}