<?php

namespace HCM\AVL\Controllers;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\AVL\Services\GerenciamentoService;
use HCM\Geral\Controllers\BirtReportController;
use HCM\Geral\Services\OperadorService;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\HttpInterface;
use HCM\Geral\Controllers\EnviarMensagemController;
use Teknisa\Model\Geral\Entities;
use HCM\Util\DateUtil;


class GerenciamentoController extends Crud {

    protected $dataSourceName = 'avl_gerenciamento';
    protected $dataSourceNameGet = 'getGerenciamento';
    protected $dataSourceNameAvaliados = 'avl_avaliadosgerenciamento';
    protected $dataSourceNameAvaliadosGet = 'getListaAvaliadosGerenciamento';
    protected $gerenciamentoService;
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    protected $operadorService;
    protected $birtReportController;
    protected $enviarMensagemController;
    protected $entityManager;
    
    const SITUACAO_CONCLUIDA = 'CONCLUIDA';
    const SITUACAO_PENDENTE  = 'PENDENTE';
    const SITUACAO_ANDAMENTO = 'ANDAMENTO';
    
    
    public function __construct ($dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, GerenciamentoService $gerenciamentoService,
        BirtReportController $birtReportController, OperadorService $operadorService, EnviarMensagemController $enviarMensagemController){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->gerenciamentoService = $gerenciamentoService;
        $this->operadorService = $operadorService;
        $this->birtReportController = $birtReportController;
        $this->enviarMensagemController = $enviarMensagemController;
        parent::__construct($dataSourceManager);
    }
    
    public function getGerenciamento(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getColaboradores(DTO\Request\Filter $request, DTO\Response $response){
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRAVALIACAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameAvaliadosGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);    
    }
    
    public function notificarPorEmail(DTO\Request\Row $row, DTO\Response $response){
        set_time_limit(0);
        ini_set('memory_limit','2048M');
        
        $message = null;
        $dataSet        = $row->getRow();
        $nrorg          = $dataSet['NRORGTRAB'];
        $mesCompetencia = $dataSet['COMPETENCIA'];
        $returnDsEmailOper = null;
        $returnAmbos       = null;
        $nravaliacao = $dataSet['NRAVALIACAO'];
        
        $avaliacaogeral = $this->gerenciamentoService->getAvaliacaoGeral($nravaliacao, $nrorg);
        $dtLimite = $avaliacaogeral->getDtfimrealizaval();
        
        
        if ((DateUtil::comparaDatas(
             DateUtil::getDataDeString($dtLimite, DateUtil::FORMATO_BRASILEIRO_DATAHORA),
             DateUtil::getDataAtual(),
             DateUtil::IGUAL))) {
                $response->addMessage(new Message('Data final da avaliação expirada. Nenhum e-mail foi enviado!', Nofitication::TYPE_WARNING));
                return;
        } elseif ($dtLimite == null) {
                $response->addMessage(new Message('Data final da avaliação não foi informada. Nenhum e-mail foi enviado!', Message::TYPE_WARNING));
                return;
        }
       
        
        if (!isset($row->getRow()['DSOBSERVACOES']))
            $row->getRow()['DSOBSERVACOES'] = '';
        else 
            $row->getRow()['DSOBSERVACOES'] = urlencode(htmlentities($row->getRow()['DSOBSERVACOES']));

        $avaliacoes = $this->gerenciamentoService->getAvaliacoes($nrorg);
        //array para tratar participantes já informados
        $nrparcnegocio_unique = array();
        $cont = 0;
        foreach ($avaliacoes as $avaliacao) {
            if ($avaliacao['SITUACAO'] == self::SITUACAO_PENDENTE || $avaliacao['SITUACAO'] == self::SITUACAO_ANDAMENTO) {
                $avaliacao['NRPARCEXECAVAL'] = (int)$avaliacao['NRPARCEXECAVAL'];
                if (!array_key_exists($avaliacao['NRPARCEXECAVAL'], $nrparcnegocio_unique)) {
                    $nrparcnegocio = $avaliacao['NRPARCEXECAVAL'];
                    
                    
                    $operador = $this->operadorService->retornaOperadorOrgParcnegocio($nrorg, $nrparcnegocio);
                    $row->getRow()['MENSAGEM'] = $this->preparaMnemonico($row->getRow()['MENSAGEM'], $avaliacao);
                    
                    $return = $this->enviarMensagemController->enviarEmail($row->getRow()['MENSAGEM'], $nrorg, $mesCompetencia, $nrparcnegocio);
                    if($return == 'DSEMAILOPER'){
                        if(!is_null($returnDsEmailOper)){
                            $returnDsEmailOper .= ' ';
                        }
                        $returnDsEmailOper.= $operador[0]['NMOPERADOR'];
                    } else if ($return == 'AMBOS'){
                        if(!is_null($returnAmbos)){
                            $returnAmbos .= ' ';
                        }
                        $returnAmbos .= $operador[0]['NMOPERADOR'];
                    }
                    $cont++;
                } else {
                    $response->addMessage(new Message(Messages::evaluateMessage(Messages::HCM03000_AVL_VINCULO_JA_NOTIFICADO, $operador[0]['NMOPERADOR']), 1), Message::TYPE_WARNING);                
                }
                //Cria key no array com o participante
                $nrparcnegocio_unique[$avaliacao['NRPARCEXECAVAL']] = $avaliacao['NRPARCEXECAVAL'];
                
                           
            }
        
        }
        
        if(!is_null($returnDsEmailOper)){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage(Messages::HCM01000_FPA_VINC_EMAIL_NOTFOUND, $returnDsEmailOper);
        }

        if(!is_null($returnAmbos)){
            if(!is_null($message)){
                $message .= '<br><br>';
            }
            $message .= Messages::evaluateMessage(Messages::HCM01000_FPA_VINC_OPERADOR_NOTFOUND. ' ou forma de comunicação', $returnAmbos);
        }
        
        if ($cont != 0) {
            $response->addMessage(new Message("$cont e-mail(s) enviado(s) com sucesso!", Message::TYPE_MESSAGE));
        } else {
            $response->addMessage(new Message("Nenhum email foi enviado.", Message::TYPE_WARNING));
        }
        
        if(!is_null($message)){
            $response->addNotification(new Notification($message, Notification::TYPE_SUCCESS));
        }
       
    }

    public function preparaMnemonico($row, $avaliacao){
        $replaceMnemonico = array(
            "[AVALIACAO]" => $avaliacao['NMAVALIACAO'], 
            "[AVALIADOR]" => $avaliacao['AVALIADOR'],
            "[DATA_INICIO]" => $avaliacao['DTINIREALIZAVAL'],
            "[DATA_FIM]" => $avaliacao['DTFIMREALIZAVAL']
        );

        $row['DSMENSAGEM'] = strtr($row['DSMENSAGEM'], $replaceMnemonico); 
        $row['DSTITULOMENSAGEM'] = strtr($row['DSTITULOMENSAGEM'], $replaceMnemonico);
        return $row;
    }

    public function reverterStatus(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $row = $rows[0];
            $this->gerenciamentoService->reverterStatus($row, $response);
            $response->addMessage(new Message("Status Revertido com sucesso", Message::TYPE_MESSAGE));
        }catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    
        
    }
    
    public function deleteRegistro(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $row = $rows[0];
            $this->gerenciamentoService->deleteRegistro($row, $response);
            $response->addMessage(new Message("Registro deletado com sucesso", Message::TYPE_MESSAGE));
        }catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
}