<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Repository\SituacaoFuncionalRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\Util\Util;
use HCM\Util\DateUtil;

use Teknisa\Model\Gestaopessoas\Entities\PtoProgocorrencia;
use Teknisa\Model\Gestaopessoas\Entities\GpeAltesitufunc;

class SituacaoFuncionalService {
    
    public const NRSITUFUNCM_ATIVIDADE_NORMAL = 1;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, SituacaoFuncionalRepository $situacaoFuncionalRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->situacaoFuncionalRepository = $situacaoFuncionalRepository;
    }

    public function retornaUltimoHistoricoCompetencia($nrorg, $nrorgPadrao, $nrsitufuncm, $dtmescompetenc) {
        try {
            return $this->situacaoFuncionalRepository->retornaUltimoHistoricoCompetencia($nrorg, $nrorgPadrao, $nrsitufuncm, $dtmescompetenc);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function deleteSituacao($dataset) {
        try {
            $this->entityManager->beginTransaction();
            $situacaoh = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCH)->findOneBy(array("nrsitufuncm" => $dataset['NRSITUFUNCM'], "nrsitufunch" => $dataset['NRSITUFUNCH'],"nrorg" => $dataset['NRORG']));
            $this->entityManager->remove($situacaoh);
            $this->entityManager->flush();
            $situacaohh = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCH)->findOneBy(array("nrsitufuncm" => $dataset['NRSITUFUNCM'],"nrorg" => $dataset['NRORG']));
            if(empty($situacaohh)) {
                $situacaom = $this->entityManager->getRepository(Repositories::GPE_SITUFUNCM)->findOneBy(array("nrsitufuncm" => $dataset['NRSITUFUNCM'],"nrorg" => $dataset['NRORG']));
                $this->entityManager->remove($situacaom);
                $this->entityManager->flush();
            }
            $this->entityManager->commit();
        } catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }
    /**
     *  Rotina responsável pelo cadastro de alteracao de situacao no momento do cadastro de ocorrencias
     * 
     * 
     *  @param $nrVinculom
     *  @param $alteredRow      array()
     *  @param $previousRow     PtoProgocorrencia
     *  @param $operation       IN [Util::DATA_OPERATION_INSERT, Util::DATA_OPERATION_UPDATE, Util::DATA_OPERATION_DELETE]
     */
    public function afterAlterOcorrenciaSetAlteSituFunc(
                            $nrVinculom,
                            $alteredRow=array(),
        PtoProgocorrencia   $previousRow=null,
                            $operation=Util::DATA_OPERATION_INSERT
    ) {
        
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        $cdOperador = $this->hcmParameters->getCdOperador();
        $dateInclusao = new \DateTime();
        
        try {
        
            $nrMotivoOcorFreq = $operation == Util::DATA_OPERATION_DELETE ? $previousRow->getNrmotivoocorfreq() ?? null : $alteredRow['NRMOTIVOOCORFREQ'] ?? null;
            if (empty($nrMotivoOcorFreq)) {
                return false;
            }
            
            // Busca pelas situacoes funcionais relacionadas ao motivo de ocorrencia cadastrado
            $associacoesMotivoSituFunc = $this->getMotivoSituFunc($nrOrg, $nrMotivoOcorFreq);
            
            if (count($associacoesMotivoSituFunc) < 1) {
                return;
            }
            
            $this->entityManager->beginTransaction();
            
            switch ($operation) {
                case Util::DATA_OPERATION_INSERT:
                    
                    $dataIniSituFuncAltered = DateUtil::getDataDeString($alteredRow['DTINICIOPROGOCOR'], DateUtil::FORMATO_BRASILEIRO, true);
                    $dataFimSituFuncAltered = !empty($alteredRow['DTFIMPROGOCOR']) ? DateUtil::getDataDeString($alteredRow['DTFIMPROGOCOR'], DateUtil::FORMATO_BRASILEIRO, true) : null;
    
                    // Insere uma alteracao de situacao funcional para cada motivo parametrizado
                    foreach ($associacoesMotivoSituFunc as $associacaoMotivoSituFunc) {
                        // Insere situacao funcional relacionada
                        $row = array(
                            'NRORG'         => $nrOrg,
                            'NRVINCULOM'    => $nrVinculom,
                            'NRSITUFUNCM'   => $associacaoMotivoSituFunc['NRSITUFUNCM'],
                            'DTINISITUFUNC' => $dataIniSituFuncAltered->format(DateUtil::FORMATO_DATA),
                            'DTFIMSITUFUNC' => empty($dataFimSituFuncAltered) ? null : $dataFimSituFuncAltered->format(DateUtil::FORMATO_DATA),
                            'CDTABECDI'     => $alteredRow['CDTABECDI']     ?? null,
                            'CDDIAGNOST'    => $alteredRow['CDDIAGNOST']    ?? null,
                            'NRPESSOA'      => $alteredRow['NRPESSOA']      ?? null,
                        );
                        
                        $this->setAlteSituacaoFuncional($row);
                        
                        // Insere situacao funcional de trabalho
                        if (!empty($dataFimSituFuncAltered)) {
                            $dataFimSituFuncAlteredProxTrab = DateUtil::adicionaIntervalo($dataFimSituFuncAltered, 1, DateUtil::DIA);
                        
                            $row = array(
                                'NRORG'         => $nrOrg,
                                'NRVINCULOM'    => $nrVinculom,
                                'NRSITUFUNCM'   => self::NRSITUFUNCM_ATIVIDADE_NORMAL,
                                'DTINISITUFUNC' => $dataFimSituFuncAlteredProxTrab->format(DateUtil::FORMATO_DATA),
                                'DTFIMSITUFUNC' => null,
                                'CDTABECDI'     => $alteredRow['CDTABECDI']     ?? null,
                                'CDDIAGNOST'    => $alteredRow['CDDIAGNOST']    ?? null,
                                'NRPESSOA'      => $alteredRow['NRPESSOA']      ?? null,
                            );
                            
                            $this->setAlteSituacaoFuncional($row);   
                        }
                    }
                    
                    break;
                case Util::DATA_OPERATION_UPDATE:
                    
                    $alteredRow['CDTABECDI']    = empty($alteredRow['CDTABECDI'])   ? null : $alteredRow['CDTABECDI'];
                    $alteredRow['CDDIAGNOST']   = empty($alteredRow['CDDIAGNOST'])  ? null : $alteredRow['CDDIAGNOST'];
                    $alteredRow['NRPESSOA']     = empty($alteredRow['NRPESSOA'])    ? null : $alteredRow['NRPESSOA'];
                    
                    $dataIniSituFuncPrevious = $previousRow->getDtinicioprogocor();
                    $dataFimSituFuncPrevious = $previousRow->getDtfimprogocor();
                    
                    $dataIniSituFuncAltered = DateUtil::getDataDeString($alteredRow['DTINICIOPROGOCOR'], DateUtil::FORMATO_DATA, true);
                    $dataFimSituFuncAltered = !empty($alteredRow['DTFIMPROGOCOR']) ? DateUtil::getDataDeString($alteredRow['DTFIMPROGOCOR'], DateUtil::FORMATO_DATA, true) : null;
                    
                    // Atualiza alteracoes de situacao funcional existentes para cada motivo parametrizado
                    foreach ($associacoesMotivoSituFunc as $associacaoMotivoSituFunc) {
                        // Altera situacao funcional relacionada
                        $alteracoesSituacaoFuncional = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(
                            array(
                                'nrvinculom'    => $nrVinculom,
                                'dtinisitufunc' => $dataIniSituFuncPrevious,
                                'dtfimsitufunc' => $dataFimSituFuncPrevious,
                                'nrsitufuncm'   => $associacaoMotivoSituFunc['NRSITUFUNCM']
                            )
                        );
                        
                        foreach($alteracoesSituacaoFuncional as $alteracaoSituacaoFuncional) {
                            $alteracaoSituacaoFuncional->setDtinisitufunc($dataIniSituFuncAltered);
                            $alteracaoSituacaoFuncional->setDtfimsitufunc($dataFimSituFuncAltered);
                            $alteracaoSituacaoFuncional->setCdtabecdi($alteredRow['CDTABECDI']      ?? null);
                            $alteracaoSituacaoFuncional->setCddiagnost($alteredRow['CDDIAGNOST']    ?? null);
                            $alteracaoSituacaoFuncional->setNrpessoa($alteredRow['NRPESSOA']        ?? null);
                            $this->entityManager->persist($alteracaoSituacaoFuncional);
                        }
                        
                        // Salva alteracoes na(s) situacao(oes) criada(s)
                        $this->entityManager->flush();
                        
                        // Altera situacao funcional de trabalho futuro
                        $dataFimSituFuncPreviousProxTrab = !empty($dataFimSituFuncPrevious) ? DateUtil::adicionaIntervalo($dataFimSituFuncPrevious, 1, DateUtil::DIA) : null;
                        $dataFimSituFuncAlteredProxTrab = !empty($dataFimSituFuncAltered) ? DateUtil::adicionaIntervalo($dataFimSituFuncAltered, 1, DateUtil::DIA) : null;
                        
                        // Verifica se é necessario criar/alterar proximas situacoes de trabalho
                        if (empty($dataFimSituFuncPreviousProxTrab) && empty($dataFimSituFuncAlteredProxTrab)) {
                            continue;
                        }
                        
                        // Verifica se devera criar a proxima situacao funcional, pois uma nova data fim foi associada a ocorrencia
                        if (empty($dataFimSituFuncPreviousProxTrab)) {
                            $row = array(
                                'NRORG'         => $nrOrg,
                                'NRVINCULOM'    => $nrVinculom,
                                'NRSITUFUNCM'   => self::NRSITUFUNCM_ATIVIDADE_NORMAL,
                                'DTINISITUFUNC' => $dataFimSituFuncAlteredProxTrab->format(DateUtil::FORMATO_DATA),
                                'DTFIMSITUFUNC' => null,
                                'CDTABECDI'     => $alteredRow['CDTABECDI']     ?? null,
                                'CDDIAGNOST'    => $alteredRow['CDDIAGNOST']    ?? null,
                                'NRPESSOA'      => $alteredRow['NRPESSOA']      ?? null,
                            );
                            
                            $this->setAlteSituacaoFuncional($row);
                        } else {
                            // Verifica se devera editar/remover a proxima situacao funcional
                            $alteracoesProxSituacaoFuncional = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(
                                array(
                                    'nrvinculom'    => $nrVinculom,
                                    'dtinisitufunc' => $dataFimSituFuncPreviousProxTrab,
                                    'dtfimsitufunc' => null,
                                    'nrsitufuncm'   => self::NRSITUFUNCM_ATIVIDADE_NORMAL
                                )
                            );
                            
                            foreach($alteracoesProxSituacaoFuncional as $alteracaoSituacaoFuncional) {
                                
                                // Caso a data fim for retirada, a situacao funcional de trabalho apos a situacao relacionada eh excluida
                                if (empty($dataFimSituFuncAlteredProxTrab)) {
                                    $this->entityManager->remove($alteracaoSituacaoFuncional);    
                                } else {
                                    // Caso contrario, a data de inicio da proxima situacao de trabalho eh somente ajustada
                                    $alteracaoSituacaoFuncional->setDtinisitufunc($dataFimSituFuncAlteredProxTrab);
                                    $this->entityManager->persist($alteracaoSituacaoFuncional);
                                }
                            }
                        }
                        
                        // Salva alteracoes realizadas na proxima situacao de trabalho
                        $this->entityManager->flush();
                    }
                    
                    break;
                case Util::DATA_OPERATION_DELETE:
                    
                    $dataIniSituFuncPrevious = $previousRow->getDtinicioprogocor();
                    $dataFimSituFuncPrevious = $previousRow->getDtfimprogocor();
                    
                    // Atualiza alteracoes de situacao funcional existentes para cada motivo parametrizado
                    foreach ($associacoesMotivoSituFunc as $associacaoMotivoSituFunc) {
                        // Remove a situacao relacionada
                        $alteracoesSituacaoFuncional = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(
                            array(
                                'nrvinculom'    => $nrVinculom,
                                'dtinisitufunc' => $dataIniSituFuncPrevious,
                                'dtfimsitufunc' => $dataFimSituFuncPrevious,
                                'nrsitufuncm'   => $associacaoMotivoSituFunc['NRSITUFUNCM']
                            )
                        );
                        
                        foreach($alteracoesSituacaoFuncional as $alteracaoSituacaoFuncional) {
                            $this->entityManager->remove($alteracaoSituacaoFuncional);
                        }
                        
                        // Remove a proxima situacao de trabalho, caso a ocorrencia possua data fim
                        if (!empty($dataFimSituFuncPrevious)) {
                            $alteracoesSituacaoFuncional = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(
                                array(
                                    'nrvinculom'    => $nrVinculom,
                                    'dtinisitufunc' => DateUtil::adicionaIntervalo($dataFimSituFuncPrevious, 1, DateUtil::DIA),
                                    'dtfimsitufunc' => null,
                                    'nrsitufuncm'   => self::NRSITUFUNCM_ATIVIDADE_NORMAL
                                )
                            );
                            
                            foreach($alteracoesSituacaoFuncional as $alteracaoSituacaoFuncional) {
                                $this->entityManager->remove($alteracaoSituacaoFuncional);
                            }
                        }
                        
                        $this->entityManager->flush();
                    }
                    break;
                default:
                    return false;
            }
            
            $this->entityManager->commit();
            return true;
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw new \Exception('Atenção! Houve um problema manipular as alterações de situação funcional. ' . $e->getMessage());
        }
    }
    
    public function getMotivoSituFunc($nrOrg, $nrMotivoOcorFreq=null, $nrSituFuncm=null) {
        return $this->situacaoFuncionalRepository->getMotivoSituFunc($nrOrg, $nrMotivoOcorFreq, $nrSituFuncm);
    }
    
    public function setAlteSituacaoFuncional(
        $row
    ){
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        $cdOperador = $this->hcmParameters->getCdOperador();
        $params = [];

        $params['NRALTESITUFUNC'] =
            $row['NRALTESITUFUNC'] ??
            \HCM\Util\NovoCodigo::geraCodigo($nrOrg, 'GPE_ALTESITUFUNC', 1, 12, true, false);

        $params['NRORG']                = $row['NRORG'] ?? $nrOrg;
        $params['NRMOTIVOAFASTA']       = $row['NRMOTIVOAFASTA'] ?? null;
        $params['NRPROXSITUFUNC']       = $row['NRPROXSITUFUNC'] ?? null;
        $params['CDTABECDI']            = $row['CDTABECDI'] ?? null;
        $params['CDDIAGNOST']           = $row['CDDIAGNOST'] ?? null;
        $params['NRSITUFUNCM']          = $row['NRSITUFUNCM'];
        $params['NRVINCULOM']           = $row['NRVINCULOM'];
        $params['DTINISITUFUNC']        = $row['DTINISITUFUNC'];
        $params['DTFIMSITUFUNC']        = $row['DTFIMSITUFUNC'] ?? null;
        $params['DTVERIFSITUFUNC']      = $row['DTVERIFSITUFUNC'] ?? null;
        $params['DTPREVIRETOSITU']      = $row['DTPREVIRETOSITU'] ?? null;
        $params['DTPRORRSITUFUNC']      = $row['DTPRORRSITUFUNC'] ?? null;
        $params['DTINCLUSAO']           = $row['DTINCLUSAO'] ?? date('d/m/Y');
        $params['DTULTATU']             = $row['DTULTATU'] ?? null;
        $params['NRORGINCLUSAO']        = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']       = $row['CDOPERINCLUSAO'] ?? $cdOperador;
        $params['NRORGULTATU']          = $row['NRORGULTATU'] ?? null;
        $params['CDOPERULTATU']         = $row['CDOPERULTATU'] ?? null;
        $params['DSOBSALTSITFUNC']      = $row['DSOBSALTSITFUNC'] ?? null;
        $params['NRQTDIASITUFUNC']      = $row['NRQTDIASITUFUNC'] ?? null;
        $params['DTESTABSITUFUNC']      = $row['DTESTABSITUFUNC'] ?? null;
        $params['NRPESSOA']             = $row['NRPESSOA'] ?? null;
        $params['QTDIASATESTADOEMPR']   = $row['QTDIASATESTADOEMPR'] ?? null;

        $this->situacaoFuncionalRepository->setAlteSituacaoFuncional($params);

        return $params['NRALTESITUFUNC'];
    }

} 