<?php
namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\PTO\Services\GerarLoteEventosService;

class GerarLoteEventosController extends Crud {

    protected $dataSourceGetGerarLoteEventos = 'getGerarLoteEventos';
    protected $dataSourceSetGerarLoteEventos = 'ptoFormGeracaoEvento_setGerarLoteEventos';
    private $criterio;
    private $inicio;
    private $fim;
    private $dtCompetencia;
    private $nrTipoVinculo;
    private $nrPeriodoApuracao;
    private $dtPeriodoIni;
    private $dtPeriodoFim;
    private $nrOrgPadrao;
    private $nrOrg;
    protected $gerarLoteEventosService;

    protected $parameterBagColumns = array('NRORG', 'DTMESCOMPETENC', 'NRORGPADRAO');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, GerarLoteEventosService $gerarLoteEventosService){
        $this->parameterBag    = $parameterBag;
        $this->gerarLoteEventosService = $gerarLoteEventosService;
        parent::__construct($dataSourceManager);
    }

    public function getGerarLoteEventos (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetGerarLoteEventos);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setGerarLoteEventos (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetGerarLoteEventos;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRORG'] = $row['NRORGTRAB'];
                    $row['NRFORMGERACAOEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PTO_FORMGERACAOEVENTO', 1, 12, true);
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }

                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }

                if(array_key_exists('DTCOMPETLOTEEVE', $row)){
                    $row['DTCOMPETLOTEEVE'] = explode(" ", $row['DTCOMPETLOTEEVE'])[0];
                    $row['DTOCORRENCIA'] = $row['DTCOMPETLOTEEVE'];
                }

                if(array_key_exists('DTLOTELANCEVENTO', $row)){
                    $row['DTLOTELANCEVENTO'] = explode(" ", $row['DTLOTELANCEVENTO'])[0];
                }

                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteGerarLoteEventos (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetGerarLoteEventos, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetGerarLoteEventos, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function geraLote(DTO\Request\DataSet $request, DTO\Response $response) {
        
        foreach($request->getDataSet()->getRows() as $row) {
            $msg = array();
            $this->criterio = $row['DSCRITERIO'];
            $this->inicio = $row['DSINICIO'];
            $this->fim = $row['DSFIM'];
            $this->dtCompetencia = $row['DTCOMPETENCIA'];
            $this->nrTipoVinculo = $row['NRTIPOVINCULOM'];
            $this->nrPeriodoApuracao = $row['NRPERIODOAPURACAO'];
            $this->dtPeriodoIni = $row['DTINICIALAPURACAO'];
            $this->dtPeriodoFim = $row['DTFINALAPURACAO'];
            $this->nrOrgPadrao = $row['NRORGPADRAO'];
            $this->nrOrg = $row['NRORGTRAB'];
        }
        $retorno = $this->gerarLoteEventosService->geraLote($this->criterio, $this->inicio, $this->fim, $this->dtCompetencia, $this->nrTipoVinculo, $this->nrPeriodoApuracao, $this->dtPeriodoIni, $this->dtPeriodoFim, $this->nrOrgPadrao, $this->nrOrg, $row['NMTIPOVINCULOH']);
        $nrlote = $retorno[0];
        if(isset($retorno[1])) {
            $msg = $retorno[1];
        }
        $response->addDataSet(new DataSet('nrlote', array($nrlote)));
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}