<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use HCM\Util\Messages;

class FichaMedicaController extends Crud {
    
    protected $fichaMedicaService;
    protected $dataSourceName = 'sst_fichamedica_get';
    protected $dataSourceName1 = 'sst_examefichamed_get';
    protected $dataSourceName2 = 'sst_itemexamefichamed_get';
    protected $dataSourceName3 = 'sst_doencafichamedica_get';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO','P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\SST\Services\FichaMedicaService $fichaMedicaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->fichaMedicaService = $fichaMedicaService;
        parent::__construct($dataSourceManager);
    }

    public function getFichaMedica(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setFichaMedica(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_fichamedica";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFICHAMEDICA', $row)) {
                    if ($row['NRFICHAMEDICA'] === '') {
                        $row['NRFICHAMEDICA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_FICHAMEDICA', 1, 12, true);
                    }
                } else {
                    $row['NRFICHAMEDICA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_FICHAMEDICA', 1, 12, true);
                }
                if(!array_key_exists('NRINFLUENCIAGERENCIAL', $row)){
                    $row['NRINFLUENCIAGERENCIAL'] = null;
                }
                if(!array_key_exists('NRNATUREZAEXAME', $row)){
                    $row['NRNATUREZAEXAME'] = null;
                }
                if(!array_key_exists('NRPARCNEGOCIO', $row)){
                    $row['NRPARCNEGOCIO'] = null;
                }
                if(!array_key_exists('NRESTRUTURAM', $row)){
                    $row['NRESTRUTURAM'] = null;
                }
                if(!array_key_exists('NRMOTIVOAUSENCIA', $row)){
                    $row['NRMOTIVOAUSENCIA'] = null;
                }
                if(!array_key_exists('NRQUEIXA', $row)){
                    $row['NRQUEIXA'] = null;
                }
                if(!array_key_exists('NRTABRESULTEXAME', $row)){
                    $row['NRTABRESULTEXAME'] = null;
                }
                if(!array_key_exists('NRCONTROLEAPTIDAOATEND', $row)){
                    $row['NRCONTROLEAPTIDAOATEND'] = null;
                }
                if(!array_key_exists('NRCONTROLEAPTIDAOINSS', $row)){
                    $row['NRCONTROLEAPTIDAOINSS'] = null;
                }
                if(!array_key_exists('NROCORRENCIA', $row)){
                    $row['NROCORRENCIA'] = null;
                }
                if(!array_key_exists('NRSITUFUNCM', $row)){
                    $row['NRSITUFUNCM'] = null;
                }
                if(!array_key_exists('NRSITUFUNCMPROX', $row)){
                    $row['NRSITUFUNCMPROX'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteFichaMedica(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->fichaMedicaService->deleteFichaMedica($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function getExameFichaMed(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setExameFichaMed(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_examefichamed";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NREXAMEFICHAMED', $row)) {
                    if ($row['NREXAMEFICHAMED'] === '') {
                        $row['NREXAMEFICHAMED'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_EXAMEFICHAMED', 1, 12, true);
                    }
                } else {
                    $row['NREXAMEFICHAMED'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_EXAMEFICHAMED', 1, 12, true);
                }
                if(!array_key_exists('NRPARCNEGOCIO', $row)){
                    $row['NRPARCNEGOCIO'] = null;
                }
                if(!array_key_exists('NRTABRESULTEXAME', $row)){
                    $row['NRTABRESULTEXAME'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteExameFichaMed(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->fichaMedicaService->deleteExameFichaMed($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function getItemExameFichaMed(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setItemExameFichaMed(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_itemexamefichamed";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRITEMEXAMEFICHAMED', $row)) {
                    if ($row['NRITEMEXAMEFICHAMED'] === '') {
                        $row['NRITEMEXAMEFICHAMED'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_ITEMEXAMEFICHAMED', 1, 12, true);
                    }
                } else {
                    $row['NRITEMEXAMEFICHAMED'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_ITEMEXAMEFICHAMED', 1, 12, true);
                }
                if(!array_key_exists('NRTABRESULTEXAME', $row)){
                    $row['NRTABRESULTEXAME'] = null;
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteItemExameFichaMed(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "sst_itemexamefichamed";
        parent::delete($request, $response);
    }
    
    public function getDoencaFichaMedica(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName3);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setDoencaFichaMedica(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_doencafichamedica";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRDOENCAFICHAMEDICA', $row)) {
                    if ($row['NRDOENCAFICHAMEDICA'] === '') {
                        $row['NRDOENCAFICHAMEDICA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_DOENCAFICHAMEDICA', 1, 12, true);
                    }
                } else {
                    $row['NRDOENCAFICHAMEDICA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'SST_DOENCAFICHAMEDICA', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteDoencaFichaMedica(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "sst_doencafichamedica";
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 