<?php

namespace Folha\Logic;

use Setforms2\Runtime\Validator\DateTime;
use HCM\Util\DateUtil;

/**
 * Class Ferias
 * @package Folha\Logic
 */
class Ferias {

    const IDCONTROLFERIAS_ABERTO = 'ABERTO';
    const IDCONTROLFERIAS_ADQUIRIDO = 'ADQUIRIDO';
    const IDCONTROLFERIAS_PERDIDO = 'PERDIDO';
    const IDCONTROLFERIAS_GOZADO_PARCIAL = 'GOZADO_PARCIAL';
    const IDCONTROLFERIAS_GOZADO_INTEGRAL = 'GOZADO_INTEGRAL';

    /**
     * @param $status
     * @param $dtiniperiodo
     * @param $dtfimperiodo
     * @param $qtdDiasFaltas
     * @param $dtinigozo
     * @param $dtfimgozo
     * @param $qtdDiasAbono
     * @return string
     */
    public static function getIdcontrolferias($status, $dtiniperiodo, $dtfimperiodo, $qtdDiasFaltas, $dtinigozo, $dtfimgozo, $qtdDiasAbono)
    {
        if (!empty($status)) {
            return $status;
        }

        $hoje = DateUtil::getDataAtual();
        $qtdDiasMaxFerias = 30;

        $dtinigozo = str_replace(" ","",$dtinigozo);
        $dtfimgozo = str_replace(" ","",$dtfimgozo);

        if ($dtinigozo != '' and $dtfimgozo != '') {
            $qtdDiasGozo = self::getQtdDiasGozo($dtinigozo, $dtfimgozo);
        } else {
            $qtdDiasGozo = 0;
        }

        if (empty($qtdDiasAbono)) {
            $qtdDiasAbono = 0;
        }

        switch ($qtdDiasMaxFerias - ($qtdDiasGozo + $qtdDiasAbono)) {
            case 0:
                $idControlFerias = self::IDCONTROLFERIAS_GOZADO_INTEGRAL;
                break;
            case 30:
                //$fim             = new \DateTime($dtfimperiodo);
                $fim = DateUtil::getDataDeString($dtfimperiodo);
                $idControlFerias = DateUtil::comparaDatas($fim, $hoje, DateUtil::DEPOIS) ? self::IDCONTROLFERIAS_ABERTO : self::IDCONTROLFERIAS_ADQUIRIDO;
                break;
            default:
                $idControlFerias = self::IDCONTROLFERIAS_GOZADO_PARCIAL;
                break;
        }

        return $idControlFerias;
    }

    /**
     * @param $dtinigozo
     * @param $dtfimgozo
     * @return int
     */
    public function getQtdDiasGozo($dtinigozo, $dtfimgozo) {
        $dtinigozo = DateUtil::getDataDeString($dtinigozo);
        $dtfimgozo = DateUtil::getDataDeString($dtfimgozo);
        //return $dtinigozo->daysAfter($dtfimgozo);
        return DateUtil::getQtdeDiasEntre($dtinigozo, $dtfimgozo) + 1;
    }


    /**
     * @param $dtfimgozo
     * @param $dtretornoferias
     * @return \TkDateTime
     */
    public static function getDtRetornoFerias($dtretornoferias, $dtfimgozo) {
        $dtfimgozo = str_replace(" ","",$dtfimgozo);
        $dtretornoferias = str_replace(" ","",$dtretornoferias);

        if (!empty($dtretornoferias)) {
            return $dtretornoferias;
        }

        if ($dtfimgozo != '') {
            $dtfimgozo = DateUtil::getDataDeString($dtfimgozo);
            $dtfimgozo = DateUtil::adicionaIntervalo($dtfimgozo);

            return $dtfimgozo;
        }
    }

    public static function existeAgencia($agencia, $banco) {

        if ($agencia != '') {
            $agencia = preg_replace("/[^0-9]/","", $agencia);
            $cdagencia     = \Geral\Entities\Agencia::getObjectByWhere('CDBANCO = ? AND CDAGENCIA = ?', array(trim($banco), trim(str_replace('-', '', $agencia))));
            $cdagencia_mig = \Folha\Entities\MigAgencia::getObjectByWhere('CDBANCO = ? AND CDAGENCIA = ?', array(trim($banco), trim(str_replace('-', '', $agencia))));

            return  (is_object($cdagencia) || is_object($cdagencia_mig)) ? false : true;
        }

        return false;
    }

    public static function dtOcorrenciaFerias($dtfimaquisicao, $cdmatricula, $nrorg) {

        $nrvinculom = \Folha\Logic\Depara::vinculo($nrorg, $cdmatricula);

        $query = "SELECT CASE TO_CHAR(MIG_GOZOFERIAS.DTINIGOZOFERIAS - 2, 'D')
                           WHEN '1' THEN '01/' || TO_CHAR(MIG_GOZOFERIAS.DTINIGOZOFERIAS - 4, 'MM/YYYY')
                           WHEN '7' THEN '01/' || TO_CHAR(MIG_GOZOFERIAS.DTINIGOZOFERIAS - 4, 'MM/YYYY')
                         ELSE '01/' || TO_CHAR(MIG_GOZOFERIAS.DTINIGOZOFERIAS - 2, 'MM/YYYY')
                         END DT_PAGAMENTO_FERIAS
                        FROM MIG_FERIAS
                        JOIN MIG_GOZOFERIAS
                        ON MIG_GOZOFERIAS.NRFERIAS                        = MIG_FERIAS.NRFERIAS
                        AND MIG_GOZOFERIAS.NRORG                          = MIG_FERIAS.NRORG
                        WHERE MIG_FERIAS.NRVINCULOM                       = '$nrvinculom'
                        AND MIG_FERIAS.NRORG                              = '$nrorg'
                        AND TO_CHAR(MIG_FERIAS.DTFIMAQUISICAO, 'MM/YYYY') = TO_CHAR(TO_DATE('$dtfimaquisicao', 'DD/MM/YYYY'), 'MM/YYYY')";

        $result = \Setforms2\Shared\Registry::getEntityManagerConnection()->exec($query);
        $data = $result->fetchColumn(0);

        if (empty($data)) {
            return $dtfimaquisicao;
        } else {
            return $data;
        }

    }

}